==> org-lsst-ccs-subsystem-focal-plane-main-1.0.3-sources.jar: org/lsst/ccs/subsystem/focalplane/ScriptingCommands.java <==
package org.lsst.ccs.subsystem.focalplane;

import java.io.File;
import java.time.Duration;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.ConcurrentSkipListSet;
import java.util.concurrent.TimeUnit;
import org.lsst.ccs.bus.states.StateBundle;
import org.lsst.ccs.command.annotations.Argument;
import org.lsst.ccs.command.annotations.Command;
import org.lsst.ccs.subsystem.focalplane.states.FocalPlaneState;

/**
 * Commands that are useful when running the focal plane subsystem with
 * scripting
 *
 * @author tonyj
 */
public class ScriptingCommands {

    private final FocalPlaneSubsystem subsys;

    ScriptingCommands(FocalPlaneSubsystem subsys) {
        this.subsys = subsys;
    }

    /**
     * Wait until all FITS files have been written.
     *
     * @param timeout The maximum duration to wait
     * @return The list of files which were written
     */
    @Command(type = Command.CommandType.QUERY, description = "Wait for FITS files to be saved", level = Command.NORMAL, autoAck = false)
    public FileList waitForFitsFiles(@Argument(defaultValue = "PT30S") Duration timeout) {
        return subsys.helper()
                .enterFaultOnException(true)
                .duration(timeout)
                .action(() -> subsys.getImageHandling().waitForFITSFiles(timeout));
    }

    /**
     * Wait until all images have been received from the DAQ (but not
     * necessarily written as FITS files)
     *
     * @param timeout The maximum duration to wait
     */
    @Command(type = Command.CommandType.QUERY, description = "Wait for image data to arrive", level = Command.NORMAL, autoAck = false)
    public void waitForImages(@Argument(defaultValue = "PT30S") Duration timeout) {
        subsys.helper()
                .enterFaultOnException(true)
                .duration(timeout)
                .action(() -> subsys.getImageHandling().waitForImages(timeout));
    }

    /**
     * Wait until the sequencer stops.
     * @param timeout 
     */
    @Command(type = Command.CommandType.QUERY, description = "Wait for sequencer to stop", level = Command.NORMAL, autoAck = false)
    public void waitForSequencer(@Argument(defaultValue = "PT30S") Duration timeout) {
        subsys.helper()
                .enterFaultOnException(true)
                .duration(timeout)
                // Note: dont use waitForStop, because that can return before the state 
                // has been updated, causing subsequent commands to fail
                .action(() -> subsys.waitFor((StateBundle t)
                -> t.isInState(FocalPlaneState.QUIESCENT) || t.isInState(FocalPlaneState.NEEDS_CLEAR),
                timeout.toMillis(), TimeUnit.MILLISECONDS));
    }

    /**
     * A simple extension of <code>List&lt;File&gt;</code> which add an extra utility method
     * for getting the common root directory of all the files in the list.
     */
    public static class FileList extends ConcurrentSkipListSet<File> {

        private static final long serialVersionUID = 3543276216049016663L;

        void addAll(List<String> files) {
            files.forEach((file) -> {
                add(new File(file));
            });
        }

        public File getCommonParentDirectory() {
            Iterator<File> iterator = this.iterator();
            if (!iterator.hasNext()) {
                return new File("/");
            }
            File result = iterator.next().getParentFile();
            while (iterator.hasNext()) {
                File file = iterator.next();
                while (!file.getAbsolutePath().startsWith(result.getAbsolutePath())) {
                    result = result.getParentFile();
                }
            }
            return result;
        }
    }
    
}

==> org-lsst-ccs-subsystem-focal-plane-main-1.0.5-sources.jar: org/lsst/ccs/subsystem/focalplane/ScriptingCommands.java <==
package org.lsst.ccs.subsystem.focalplane;

import java.io.File;
import java.time.Duration;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.ConcurrentSkipListSet;
import java.util.concurrent.TimeUnit;
import org.lsst.ccs.bus.states.StateBundle;
import org.lsst.ccs.command.annotations.Argument;
import org.lsst.ccs.command.annotations.Command;
import org.lsst.ccs.services.AgentStateService;
import org.lsst.ccs.subsystem.focalplane.states.FocalPlaneState;

/**
 * Commands that are useful when running the focal plane subsystem with
 * scripting
 *
 * @author tonyj
 */
public class ScriptingCommands {

    private final FocalPlaneSubsystem subsys;

    ScriptingCommands(FocalPlaneSubsystem subsys) {
        this.subsys = subsys;
    }

    /**
     * Wait until all FITS files have been written.
     *
     * @param timeout The maximum duration to wait
     * @return The list of files which were written
     */
    @Command(type = Command.CommandType.QUERY, description = "Wait for FITS files to be saved", level = Command.NORMAL, autoAck = false)
    public FileList waitForFitsFiles(@Argument(defaultValue = "PT30S") Duration timeout) {
        return subsys.helper()
                .enterFaultOnException(true)
                .duration(timeout)
                .action(() -> subsys.getImageHandling().waitForFITSFiles(timeout));
    }

    /**
     * Wait until all images have been received from the DAQ (but not
     * necessarily written as FITS files)
     *
     * @param timeout The maximum duration to wait
     */
    @Command(type = Command.CommandType.QUERY, description = "Wait for image data to arrive", level = Command.NORMAL, autoAck = false)
    public void waitForImages(@Argument(defaultValue = "PT30S") Duration timeout) {
        subsys.helper()
                .enterFaultOnException(true)
                .duration(timeout)
                .action(() -> subsys.getImageHandling().waitForImages(timeout));
    }

    /**
     * Wait until the sequencer stops.
     * @param timeout 
     */
    @Command(type = Command.CommandType.QUERY, description = "Wait for sequencer to stop", level = Command.NORMAL, autoAck = false)
    public void waitForSequencer(@Argument(defaultValue = "PT30S") Duration timeout) {
        subsys.helper()
                .enterFaultOnException(true)
                .duration(timeout)
                // Note: dont use waitForStop, because that can return before the state 
                // has been updated, causing subsequent commands to fail
                .action(() -> subsys.getAgentService(AgentStateService.class).waitFor((StateBundle t)
                -> t.isInState(FocalPlaneState.QUIESCENT) || t.isInState(FocalPlaneState.NEEDS_CLEAR),
                timeout.toMillis(), TimeUnit.MILLISECONDS));
    }

    /**
     * A simple extension of <code>List&lt;File&gt;</code> which add an extra utility method
     * for getting the common root directory of all the files in the list.
     */
    public static class FileList extends ConcurrentSkipListSet<File> {

        private static final long serialVersionUID = 3543276216049016663L;

        void addAll(List<String> files) {
            files.forEach((file) -> {
                add(new File(file));
            });
        }

        public File getCommonParentDirectory() {
            Iterator<File> iterator = this.iterator();
            if (!iterator.hasNext()) {
                return new File("/");
            }
            File result = iterator.next().getParentFile();
            while (iterator.hasNext()) {
                File file = iterator.next();
                while (!file.getAbsolutePath().startsWith(result.getAbsolutePath())) {
                    result = result.getParentFile();
                }
            }
            return result;
        }
    }
    
}
